@@ -526,9 +526,16 @@ func (dc *downstreamConn) SendMessage(ctx context.Context, msg *irc.Message) {
 	}
 	if msg.Prefix != nil && msg.Prefix.Name == "*" {
 		// We use "*" as a sentinel value to simplify upstream message handling
-		msg = msg.Copy()
+		msgCopy := *msg
+		msg = &msgCopy
 		msg.Prefix = nil
 	}
+	if msg.Prefix == nil && isNumeric(msg.Command) {
+		// Numerics must always carry a source
+		msgCopy := *msg
+		msg = &msgCopy
+		msg.Prefix = dc.srv.prefix()
+	}
 
 	dc.srv.metrics.downstreamOutMessagesTotal.Inc()
 	dc.conn.SendMessage(ctx, msg)
@@ -541,7 +548,6 @@ func (dc *downstreamConn) SendBatch(ctx context.Context, typ string, params []st
 	if dc.caps.IsEnabled("batch") {
 		dc.SendMessage(ctx, &irc.Message{
 			Tags:    tags,
-			Prefix:  dc.srv.prefix(),
 			Command: "BATCH",
 			Params:  append([]string{"+" + ref, typ}, params...),
 		})
@@ -551,7 +557,6 @@ func (dc *downstreamConn) SendBatch(ctx context.Context, typ string, params []st
 
 	if dc.caps.IsEnabled("batch") {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: "BATCH",
 			Params:  []string{"-" + ref},
 		})
@@ -622,7 +627,8 @@ func (dc *downstreamConn) handleMessage(ctx context.Context, msg *irc.Message) e
 
 	switch msg.Command {
 	case "QUIT":
-		return dc.Close()
+		dc.conn.Shutdown(ctx)
+		return nil // TODO: stop handling commands
 	default:
 		if dc.registered {
 			return dc.handleMessageRegistered(ctx, msg)
@@ -695,7 +701,6 @@ func (dc *downstreamConn) handleMessageUnregistered(ctx context.Context, msg *ir
 		if err != nil {
 			dc.logger.Printf("SASL %v authentication error for nick %q: %v", credentials.mechanism, dc.registration.nick, err)
 			dc.endSASL(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, authErrorReason(err)},
 			})
@@ -903,21 +908,18 @@ func (dc *downstreamConn) handleAuthenticate(ctx context.Context, msg *irc.Messa
 
 	if !dc.caps.IsEnabled("sasl") {
 		return nil, ircError{&irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.ERR_SASLFAIL,
 			Params:  []string{dc.nick, "AUTHENTICATE requires the \"sasl\" capability to be enabled"},
 		}}
 	}
 	if len(msg.Params) == 0 {
 		return nil, ircError{&irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.ERR_SASLFAIL,
 			Params:  []string{dc.nick, "Missing AUTHENTICATE argument"},
 		}}
 	}
 	if msg.Params[0] == "*" {
 		return nil, ircError{&irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.ERR_SASLABORTED,
 			Params:  []string{dc.nick, "SASL authentication aborted"},
 		}}
@@ -950,7 +952,6 @@ func (dc *downstreamConn) handleAuthenticate(ctx context.Context, msg *irc.Messa
 			})
 		default:
 			return nil, ircError{&irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, "Unsupported SASL mechanism"},
 			}}
@@ -965,7 +966,6 @@ func (dc *downstreamConn) handleAuthenticate(ctx context.Context, msg *irc.Messa
 
 		if dc.sasl.pendingResp.Len()+len(chunk) > 10*1024 {
 			return nil, ircError{&irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, "Response too long"},
 			}}
@@ -980,7 +980,6 @@ func (dc *downstreamConn) handleAuthenticate(ctx context.Context, msg *irc.Messa
 		resp, err = base64.StdEncoding.DecodeString(dc.sasl.pendingResp.String())
 		if err != nil {
 			return nil, ircError{&irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, "Invalid base64-encoded response"},
 			}}
@@ -1002,7 +1001,6 @@ func (dc *downstreamConn) handleAuthenticate(ctx context.Context, msg *irc.Messa
 
 		// TODO: multi-line messages
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: "AUTHENTICATE",
 			Params:  []string{challengeStr},
 		})
@@ -1021,7 +1019,6 @@ func (dc *downstreamConn) endSASL(ctx context.Context, msg *irc.Message) {
 		dc.SendMessage(ctx, msg)
 	} else {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_SASLSUCCESS,
 			Params:  []string{dc.nick, "SASL authentication successful"},
 		})
@@ -1202,13 +1199,11 @@ func (dc *downstreamConn) updateAccount(ctx context.Context) {
 
 	if account != "" {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_LOGGEDIN,
 			Params:  []string{dc.nick, dc.prefix().String(), account, "You are logged in as " + account},
 		})
 	} else {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_LOGGEDOUT,
 			Params:  []string{dc.nick, dc.prefix().String(), "You are logged out"},
 		})
@@ -1279,7 +1274,6 @@ func (dc *downstreamConn) register(ctx context.Context) error {
 
 	if dc.sasl != nil {
 		dc.endSASL(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.ERR_SASLABORTED,
 			Params:  []string{dc.nick, "SASL authentication aborted"},
 		})
@@ -1472,6 +1466,9 @@ func (dc *downstreamConn) welcome(ctx context.Context, user *user) error {
 	if dc.caps.IsEnabled("soju.im/webpush") {
 		isupport = append(isupport, "VAPID="+dc.srv.webPush.VAPIDKeys.Public)
 	}
+	if dc.srv.Config().FileUploader != nil {
+		isupport = append(isupport, "soju.im/FILEHOST="+dc.srv.Config().HTTPIngress+"/uploads")
+	}
 
 	if uc := dc.upstream(); uc != nil {
 		// If upstream doesn't support message-tags, indicate that we'll drop
@@ -1494,33 +1491,28 @@ func (dc *downstreamConn) welcome(ctx context.Context, user *user) error {
 	}
 
 	dc.SendMessage(ctx, &irc.Message{
-		Prefix:  dc.srv.prefix(),
 		Command: irc.RPL_WELCOME,
 		Params:  []string{dc.nick, "Welcome to soju, " + dc.nick},
 	})
 	dc.SendMessage(ctx, &irc.Message{
-		Prefix:  dc.srv.prefix(),
 		Command: irc.RPL_YOURHOST,
 		Params:  []string{dc.nick, "Your host is " + dc.srv.Config().Hostname},
 	})
 	dc.SendMessage(ctx, &irc.Message{
-		Prefix:  dc.srv.prefix(),
 		Command: irc.RPL_MYINFO,
 		Params:  []string{dc.nick, dc.srv.Config().Hostname, "soju", "aiwroO", "OovaimnqpsrtklbeI"},
 	})
-	for _, msg := range xirc.GenerateIsupport(dc.srv.prefix(), isupport) {
+	for _, msg := range xirc.GenerateIsupport(isupport) {
 		dc.SendMessage(ctx, msg)
 	}
 	if uc := dc.upstream(); uc != nil {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_UMODEIS,
 			Params:  []string{dc.nick, "+" + string(uc.modes)},
 		})
 	}
 	if dc.network == nil && dc.user.Admin {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_UMODEIS,
 			Params:  []string{dc.nick, "+o"},
 		})
@@ -1532,7 +1524,7 @@ func (dc *downstreamConn) welcome(ctx context.Context, user *user) error {
 	dc.updateCasemapping()
 
 	if motd := dc.user.srv.Config().MOTD; motd != "" && dc.network == nil {
-		for _, msg := range xirc.GenerateMOTD(dc.srv.prefix(), motd) {
+		for _, msg := range xirc.GenerateMOTD(motd) {
 			dc.SendMessage(ctx, msg)
 		}
 	} else {
@@ -1541,7 +1533,6 @@ func (dc *downstreamConn) welcome(ctx context.Context, user *user) error {
 			motdHint = "Use /motd to read the message of the day"
 		}
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.ERR_NOMOTD,
 			Params:  []string{dc.nick, motdHint},
 		})
@@ -1554,7 +1545,6 @@ func (dc *downstreamConn) welcome(ctx context.Context, user *user) error {
 				attrs := getNetworkAttrs(network)
 				dc.SendMessage(ctx, &irc.Message{
 					Tags:    irc.Tags{"batch": batchRef},
-					Prefix:  dc.srv.prefix(),
 					Command: "BOUNCER",
 					Params:  []string{"NETWORK", idStr, attrs.String()},
 				})
@@ -1690,12 +1680,15 @@ func (dc *downstreamConn) runUntilRegistered() error {
 	go func() {
 		<-ctx.Done()
 		if err := ctx.Err(); err == context.DeadlineExceeded {
+			ctx, cancel := context.WithTimeout(context.Background(), 3*time.Second)
+			defer cancel()
+
 			dc.SendMessage(ctx, &irc.Message{
 				Prefix:  dc.srv.prefix(),
 				Command: "ERROR",
 				Params:  []string{"Connection registration timed out"},
 			})
-			dc.Close()
+			dc.Shutdown(ctx)
 		}
 	}()
 
@@ -1895,7 +1888,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 
 			if name == "" || strings.ContainsAny(name, illegalChanChars) {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.ERR_BADCHANMASK,
 					Params:  []string{name, "Invalid channel name"},
 				})
@@ -1903,7 +1895,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 			if !uc.isChannel(name) {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.ERR_NOSUCHCHANNEL,
 					Params:  []string{name, "Not a channel name"},
 				})
@@ -2024,7 +2015,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				}
 
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_UMODEIS,
 					Params:  []string{dc.nick, "+" + userMode},
 				})
@@ -2070,13 +2060,11 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			params = append(params, modeParams...)
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_CHANNELMODEIS,
 				Params:  params,
 			})
 			if ch.creationTime != "" {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: xirc.RPL_CREATIONTIME,
 					Params:  []string{dc.nick, name, ch.creationTime},
 				})
@@ -2123,7 +2111,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 
 		if len(msg.Params) == 0 {
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFNAMES,
 				Params:  []string{dc.nick, "*", "End of /NAMES list"},
 			})
@@ -2146,7 +2133,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 	case "WHO":
 		if len(msg.Params) == 0 {
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHO,
 				Params:  []string{"*", "*", "End of /WHO list"},
 			})
@@ -2183,9 +2169,8 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				Account:  dc.user.Username,
 				Realname: dc.realname,
 			}
-			dc.SendMessage(ctx, xirc.GenerateWHOXReply(dc.srv.prefix(), fields, &info))
+			dc.SendMessage(ctx, xirc.GenerateWHOXReply(fields, &info))
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHO,
 				Params:  []string{"*", endOfWhoToken, "End of /WHO list"},
 			})
@@ -2210,9 +2195,8 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				Account:  serviceNick,
 				Realname: serviceRealname,
 			}
-			dc.SendMessage(ctx, xirc.GenerateWHOXReply(dc.srv.prefix(), fields, &info))
+			dc.SendMessage(ctx, xirc.GenerateWHOXReply(fields, &info))
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHO,
 				Params:  []string{"*", endOfWhoToken, "End of /WHO list"},
 			})
@@ -2220,7 +2204,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 		}
 		if dc.network == nil {
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHO,
 				Params:  []string{"*", endOfWhoToken, "End of /WHO list"},
 			})
@@ -2248,10 +2231,9 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				if uc.isChannel(mask) {
 					info.Channel = mask
 				}
-				dc.SendMessage(ctx, xirc.GenerateWHOXReply(dc.srv.prefix(), fields, &info))
+				dc.SendMessage(ctx, xirc.GenerateWHOXReply(fields, &info))
 			}
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHO,
 				Params:  []string{"*", endOfWhoToken, "End of /WHO list"},
 			})
@@ -2280,29 +2262,24 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 
 		if dc.network == nil && dc.casemap(mask) == dc.nickCM {
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_WHOISUSER,
 				Params:  []string{dc.nick, dc.nick, dc.user.Username, dc.hostname, "*", dc.realname},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_WHOISSERVER,
 				Params:  []string{dc.nick, dc.nick, dc.srv.Config().Hostname, "soju"},
 			})
 			if dc.user.Admin {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_WHOISOPERATOR,
 					Params:  []string{dc.nick, dc.nick, "is a bouncer administrator"},
 				})
 			}
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: xirc.RPL_WHOISACCOUNT,
 				Params:  []string{dc.nick, dc.nick, dc.user.Username, "is logged in as"},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHOIS,
 				Params:  []string{dc.nick, dc.nick, "End of /WHOIS list"},
 			})
@@ -2310,32 +2287,26 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 		}
 		if dc.casemap(mask) == serviceNickCM {
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_WHOISUSER,
 				Params:  []string{dc.nick, serviceNick, servicePrefix.User, servicePrefix.Host, "*", serviceRealname},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_WHOISSERVER,
 				Params:  []string{dc.nick, serviceNick, dc.srv.Config().Hostname, "soju"},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_WHOISOPERATOR,
 				Params:  []string{dc.nick, serviceNick, "is the bouncer service"},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: xirc.RPL_WHOISACCOUNT,
 				Params:  []string{dc.nick, serviceNick, serviceNick, "is logged in as"},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: xirc.RPL_WHOISBOT,
 				Params:  []string{dc.nick, serviceNick, "is a bot"},
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFWHOIS,
 				Params:  []string{dc.nick, serviceNick, "End of /WHOIS list"},
 			})
@@ -2373,7 +2344,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				// hostname, broadcast the message to all bouncer users.
 				if !dc.user.Admin {
 					return ircError{&irc.Message{
-						Prefix:  dc.srv.prefix(),
 						Command: irc.ERR_BADMASK,
 						Params:  []string{dc.nick, name, "Permission denied to broadcast message to all bouncer users"},
 					}}
@@ -2510,7 +2480,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 
 		if uc.saslClient != nil {
 			dc.endSASL(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, "Another authentication attempt is already in progress"},
 			})
@@ -2537,7 +2506,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				if err != nil {
 					dc.logger.Printf("failed to clear SASL credentials")
 					dc.endSASL(ctx, &irc.Message{
-						Prefix:  dc.srv.prefix(),
 						Command: irc.ERR_SASLFAIL,
 						Params:  []string{dc.nick, "Internal server error"},
 					})
@@ -2547,7 +2515,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			dc.endSASL(ctx, nil)
 		default:
 			dc.endSASL(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.ERR_SASLFAIL,
 				Params:  []string{dc.nick, "Unsupported SASL authentication mechanism"},
 			})
@@ -2628,7 +2595,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 					// Hard limit, just to avoid having downstreams fill our map
 					if dc.monitored.Len() >= 1000 {
 						dc.SendMessage(ctx, &irc.Message{
-							Prefix:  dc.srv.prefix(),
 							Command: irc.ERR_MONLISTFULL,
 							Params:  []string{dc.nick, "1000", target, "Bouncer monitor list is full"},
 						})
@@ -2640,7 +2606,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 					if uc.network.casemap(target) == serviceNickCM {
 						// BouncerServ is never tired
 						dc.SendMessage(ctx, &irc.Message{
-							Prefix:  dc.srv.prefix(),
 							Command: irc.RPL_MONONLINE,
 							Params:  []string{dc.nick, target},
 						})
@@ -2654,7 +2619,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 						}
 
 						dc.SendMessage(ctx, &irc.Message{
-							Prefix:  dc.srv.prefix(),
 							Command: cmd,
 							Params:  []string{dc.nick, target},
 						})
@@ -2671,13 +2635,11 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			// TODO: be less lazy and pack the list
 			dc.monitored.ForEach(func(name string, _ struct{}) {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_MONLIST,
 					Params:  []string{dc.nick, name},
 				})
 			})
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: irc.RPL_ENDOFMONLIST,
 				Params:  []string{dc.nick, "End of MONITOR list"},
 			})
@@ -2694,7 +2656,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 				}
 
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: cmd,
 					Params:  []string{dc.nick, target},
 				})
@@ -2824,7 +2785,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 
 					dc.SendMessage(ctx, &irc.Message{
 						Tags:    irc.Tags{"batch": batchRef},
-						Prefix:  dc.srv.prefix(),
 						Command: "CHATHISTORY",
 						Params:  []string{"TARGETS", target.Name, xirc.FormatServerTime(target.LatestMessage)},
 					})
@@ -3056,7 +3016,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 					attrs := getNetworkAttrs(network)
 					dc.SendMessage(ctx, &irc.Message{
 						Tags:    irc.Tags{"batch": batchRef},
-						Prefix:  dc.srv.prefix(),
 						Command: "BOUNCER",
 						Params:  []string{"NETWORK", idStr, attrs.String()},
 					})
@@ -3091,7 +3050,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: "BOUNCER",
 				Params:  []string{"ADDNETWORK", fmt.Sprintf("%v", network.ID)},
 			})
@@ -3135,7 +3093,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: "BOUNCER",
 				Params:  []string{"CHANGENETWORK", idStr},
 			})
@@ -3162,7 +3119,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: "BOUNCER",
 				Params:  []string{"DELNETWORK", idStr},
 			})
@@ -3266,7 +3222,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: "WEBPUSH",
 				Params:  []string{"REGISTER", endpoint},
 			})
@@ -3288,7 +3243,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			oldSub := findWebPushSubscription(subs, endpoint)
 			if oldSub == nil {
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: "WEBPUSH",
 					Params:  []string{"UNREGISTER", endpoint},
 				})
@@ -3304,7 +3258,6 @@ func (dc *downstreamConn) handleMessageRegistered(ctx context.Context, msg *irc.
 			}
 
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: "WEBPUSH",
 				Params:  []string{"UNREGISTER", endpoint},
 			})
@@ -3432,21 +3385,18 @@ func forwardChannel(ctx context.Context, dc *downstreamConn, ch *upstreamChannel
 func sendTopic(ctx context.Context, dc *downstreamConn, ch *upstreamChannel) {
 	if ch.Topic != "" {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_TOPIC,
 			Params:  []string{dc.nick, ch.Name, ch.Topic},
 		})
 		if ch.TopicWho != nil {
 			topicTime := strconv.FormatInt(ch.TopicTime.Unix(), 10)
 			dc.SendMessage(ctx, &irc.Message{
-				Prefix:  dc.srv.prefix(),
 				Command: xirc.RPL_TOPICWHOTIME,
 				Params:  []string{dc.nick, ch.Name, ch.TopicWho.String(), topicTime},
 			})
 		}
 	} else {
 		dc.SendMessage(ctx, &irc.Message{
-			Prefix:  dc.srv.prefix(),
 			Command: irc.RPL_NOTOPIC,
 			Params:  []string{dc.nick, ch.Name, "No topic is set"},
 		})
@@ -3460,7 +3410,7 @@ func sendNames(ctx context.Context, dc *downstreamConn, ch *upstreamChannel) {
 		members = append(members, s)
 	})
 
-	msgs := xirc.GenerateNamesReply(dc.srv.prefix(), ch.Name, ch.Status, members)
+	msgs := xirc.GenerateNamesReply(ch.Name, ch.Status, members)
 	for _, msg := range msgs {
 		dc.SendMessage(ctx, msg)
 	}
