@@ -23,6 +23,7 @@ import (
 	"git.sr.ht/~emersion/soju/auth"
 	"git.sr.ht/~emersion/soju/config"
 	"git.sr.ht/~emersion/soju/database"
+	"git.sr.ht/~emersion/soju/fileupload"
 	"git.sr.ht/~emersion/soju/identd"
 )
 
@@ -142,6 +143,7 @@ type Config struct {
 	MsgStoreDriver            string
 	MsgStorePath              string
 	HTTPOrigins               []string
+	HTTPIngress               string
 	AcceptProxyIPs            config.IPSet
 	MaxUserNetworks           int
 	MOTD                      string
@@ -149,6 +151,7 @@ type Config struct {
 	DisableInactiveUsersDelay time.Duration
 	EnableUsersOnAuth         bool
 	Auth                      auth.Authenticator
+	FileUploader              fileupload.Uploader
 }
 
 type Server struct {
@@ -339,10 +342,12 @@ func (s *Server) sendWebPush(ctx context.Context, sub *webpush.Subscription, vap
 		},
 		VAPIDPublicKey:  s.webPush.VAPIDKeys.Public,
 		VAPIDPrivateKey: s.webPush.VAPIDKeys.Private,
-		Subscriber:      "https://soju.im",
-		TTL:             7 * 24 * 60 * 60, // seconds
-		Urgency:         urgency,
-		RecordSize:      2048,
+		// TODO: switch back to an HTTP URL once this is merged:
+		// https://github.com/SherClockHolmes/webpush-go/pull/57
+		Subscriber: "webpush@soju.im",
+		TTL:        7 * 24 * 60 * 60, // seconds
+		Urgency:    urgency,
+		RecordSize: 2048,
 	}
 
 	if vapidPubKey != options.VAPIDPublicKey {
@@ -467,7 +472,7 @@ func (s *Server) Handle(ic ircConn) {
 
 	id := atomic.AddUint64(&lastDownstreamID, 1)
 	dc := newDownstreamConn(s, ic, id)
-	defer dc.Close()
+	defer dc.Shutdown(context.TODO())
 
 	if shutdown {
 		dc.SendMessage(context.TODO(), &irc.Message{
@@ -486,6 +491,7 @@ func (s *Server) Handle(ic ircConn) {
 
 	user, err := s.getOrCreateUser(context.TODO(), dc.registration.authUsername)
 	if err != nil {
+		dc.logger.Printf("failed to get/create user: %v", err)
 		dc.SendMessage(context.TODO(), &irc.Message{
 			Command: "ERROR",
 			Params:  []string{"Internal server error"},
