@@ -12,6 +12,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"math/rand"
 	"net"
 	"strconv"
 	"strings"
@@ -242,8 +243,6 @@ func connectToUpstream(ctx context.Context, network *network) (*upstreamConn, er
 	ctx, cancel := context.WithTimeout(ctx, connectTimeout)
 	defer cancel()
 
-	var dialer net.Dialer
-
 	u, err := network.URL()
 	if err != nil {
 		return nil, err
@@ -259,13 +258,6 @@ func connectToUpstream(ctx context.Context, network *network) (*upstreamConn, er
 			addr = u.Host + ":6697"
 		}
 
-		dialer.LocalAddr, err = network.user.localTCPAddrForHost(ctx, host)
-		if err != nil {
-			return nil, fmt.Errorf("failed to pick local IP for remote host %q: %v", host, err)
-		}
-
-		logger.Printf("connecting to TLS server at address %q", addr)
-
 		tlsConfig := &tls.Config{ServerName: host, NextProtos: []string{"irc"}}
 		if network.SASL.Mechanism == "EXTERNAL" {
 			if network.SASL.External.CertBlob == nil {
@@ -321,9 +313,10 @@ func connectToUpstream(ctx context.Context, network *network) (*upstreamConn, er
 			}
 		}
 
-		netConn, err = dialer.DialContext(ctx, "tcp", addr)
+		logger.Printf("connecting to TLS server at address %q", addr)
+		netConn, err = dialTCP(ctx, network.user, addr)
 		if err != nil {
-			return nil, fmt.Errorf("failed to dial %q: %v", addr, err)
+			return nil, err
 		}
 
 		// Don't do the TLS handshake immediately, because we need to register
@@ -332,23 +325,17 @@ func connectToUpstream(ctx context.Context, network *network) (*upstreamConn, er
 		netConn = tls.Client(netConn, tlsConfig)
 	case "irc+insecure":
 		addr := u.Host
-		host, _, err := net.SplitHostPort(addr)
-		if err != nil {
-			host = u.Host
+		if _, _, err := net.SplitHostPort(addr); err != nil {
 			addr = u.Host + ":6667"
 		}
 
-		dialer.LocalAddr, err = network.user.localTCPAddrForHost(ctx, host)
-		if err != nil {
-			return nil, fmt.Errorf("failed to pick local IP for remote host %q: %v", host, err)
-		}
-
 		logger.Printf("connecting to plain-text server at address %q", addr)
-		netConn, err = dialer.DialContext(ctx, "tcp", addr)
+		netConn, err = dialTCP(ctx, network.user, addr)
 		if err != nil {
-			return nil, fmt.Errorf("failed to dial %q: %v", addr, err)
+			return nil, err
 		}
 	case "irc+unix", "unix":
+		var dialer net.Dialer
 		logger.Printf("connecting to Unix socket at path %q", u.Path)
 		netConn, err = dialer.DialContext(ctx, "unix", u.Path)
 		if err != nil {
@@ -386,6 +373,32 @@ func connectToUpstream(ctx context.Context, network *network) (*upstreamConn, er
 	return uc, nil
 }
 
+func dialTCP(ctx context.Context, user *user, addr string) (net.Conn, error) {
+	var dialer net.Dialer
+	upstreamUserIPs := user.srv.Config().UpstreamUserIPs
+	if len(upstreamUserIPs) > 0 {
+		host, port, err := net.SplitHostPort(addr)
+		if err != nil {
+			return nil, err
+		}
+
+		ipAddr, err := resolveIPAddr(ctx, host)
+		if err != nil {
+			return nil, fmt.Errorf("failed to resolve host %q: %v", host, err)
+		}
+
+		localAddr, err := user.localTCPAddr(ipAddr.IP)
+		if err != nil {
+			return nil, fmt.Errorf("failed to pick local IP for remote host %q: %v", host, err)
+		}
+
+		addr = net.JoinHostPort(ipAddr.String(), port)
+		dialer.LocalAddr = localAddr
+	}
+
+	return dialer.DialContext(ctx, "tcp", addr)
+}
+
 func (uc *upstreamConn) forEachDownstream(f func(*downstreamConn)) {
 	uc.network.forEachDownstream(f)
 }
@@ -448,7 +461,6 @@ func (uc *upstreamConn) abortPendingCommands() {
 			switch pendingCmd.msg.Command {
 			case "LIST":
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_LISTEND,
 					Params:  []string{dc.nick, "Command aborted"},
 				})
@@ -458,26 +470,22 @@ func (uc *upstreamConn) abortPendingCommands() {
 					mask = pendingCmd.msg.Params[0]
 				}
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_ENDOFWHO,
 					Params:  []string{dc.nick, mask, "Command aborted"},
 				})
 			case "WHOIS":
 				nick := pendingCmd.msg.Params[len(pendingCmd.msg.Params)-1]
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.RPL_ENDOFWHOIS,
 					Params:  []string{dc.nick, nick, "Command aborted"},
 				})
 			case "AUTHENTICATE":
 				dc.endSASL(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: irc.ERR_SASLABORTED,
 					Params:  []string{dc.nick, "SASL authentication aborted"},
 				})
 			case "REGISTER", "VERIFY":
 				dc.SendMessage(ctx, &irc.Message{
-					Prefix:  dc.srv.prefix(),
 					Command: "FAIL",
 					Params:  []string{pendingCmd.msg.Command, "TEMPORARILY_UNAVAILABLE", pendingCmd.msg.Params[0], "Command aborted"},
 				})
@@ -1004,7 +1012,7 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 		uc.updateMonitor()
 
 		uc.forEachDownstream(func(dc *downstreamConn) {
-			msgs := xirc.GenerateIsupport(dc.srv.prefix(), downstreamIsupport)
+			msgs := xirc.GenerateIsupport(downstreamIsupport)
 			for _, msg := range msgs {
 				dc.SendMessage(ctx, msg)
 			}
@@ -1376,8 +1384,10 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 			return err
 		}
 		modeStr := ""
+		var modeArgs []string
 		if len(msg.Params) > 2 {
 			modeStr = msg.Params[2]
+			modeArgs = msg.Params[3:]
 		}
 
 		ch := uc.channels.Get(channel)
@@ -1388,7 +1398,7 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 
 		firstMode := ch.modes == nil
 		ch.modes = make(map[byte]string)
-		if err := applyChannelModes(ch, modeStr, msg.Params[3:]); err != nil {
+		if err := applyChannelModes(ch, modeStr, modeArgs); err != nil {
 			return err
 		}
 
@@ -1603,6 +1613,18 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 
 		weAreInvited := uc.isOurNick(nick)
 
+		if weAreInvited {
+			joined := uc.channels.Get(channel) != nil
+			c := uc.network.channels.Get(channel)
+			if !joined && c != nil {
+				// Automatically join a saved channel when we are invited
+				for _, msg := range xirc.GenerateJoin([]string{c.Name}, []string{c.Key}) {
+					uc.SendMessage(ctx, msg)
+				}
+				break
+			}
+		}
+
 		uc.forEachDownstream(func(dc *downstreamConn) {
 			if !weAreInvited && !dc.caps.IsEnabled("invite-notify") {
 				return
@@ -1658,7 +1680,6 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 				prefix := irc.ParsePrefix(target)
 				if dc.monitored.Has(prefix.Name) {
 					dc.SendMessage(ctx, &irc.Message{
-						Prefix:  dc.srv.prefix(),
 						Command: msg.Command,
 						Params:  []string{dc.nick, target},
 					})
@@ -1676,7 +1697,6 @@ func (uc *upstreamConn) handleMessage(ctx context.Context, msg *irc.Message) err
 			for _, target := range targets {
 				if dc.monitored.Has(target) {
 					dc.SendMessage(ctx, &irc.Message{
-						Prefix:  dc.srv.prefix(),
 						Command: msg.Command,
 						Params:  []string{dc.nick, limit, target},
 					})
@@ -2435,3 +2455,31 @@ func (uc *upstreamConn) shouldCacheUserInfo(nick string) bool {
 	})
 	return found
 }
+
+// resolveIPAddr replaces the standard library's DNS resolver to randomize the
+// result order instead of always returning the same IP address. The bouncer
+// will often have bursts of connections to the same host (e.g. on startup) so
+// it's more important for our use-case to distribute the traffic among
+// available IP addresses than to find the fastest link.
+//
+// See: https://todo.sr.ht/~emersion/soju/221
+func resolveIPAddr(ctx context.Context, host string) (*net.IPAddr, error) {
+	ipAddrs, err := net.DefaultResolver.LookupIPAddr(ctx, host)
+	if err != nil {
+		return nil, err
+	}
+
+	// Prefer IPv6 if available, for per-user local IP addresses
+	ip6Addrs := make([]net.IPAddr, 0, len(ipAddrs))
+	for _, ipAddr := range ipAddrs {
+		if ipAddr.IP.To4() == nil {
+			ip6Addrs = append(ip6Addrs, ipAddr)
+		}
+	}
+	if len(ip6Addrs) > 0 {
+		ipAddrs = ip6Addrs
+	}
+
+	i := rand.Intn(len(ipAddrs))
+	return &ipAddrs[i], nil
+}
